unit uOptions;
{$INCLUDE 'DirSync3.inc'}

interface

uses
  SysUtils,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  uDirSyncConsts;

type
  TDirSyncOptions = class(TFarConfig)
  private
    fChanged: boolean;
    fProcessSubfolders: boolean;
    fUseMaxScanDepth: boolean;
    fMaxScanDepth: integer;
    fProcessSelected: boolean;
    fCompareTime: boolean;
    fLowPrecisionTime: boolean;
    fIgnorePossibleTimeZoneDifferences: boolean;
    fCompareSize: boolean;
    fCompareFileNameCase: boolean;
    fCompareContents: boolean;
    fCompareContentsIgnore: boolean;
    fIgnoreNewLines: boolean;
    fIgnoreWhitespace: boolean;
    fSelectDifferencesOnBothPanels: boolean;
    fCompareCommand: string;
    fCompareCommandArguments: string;
    fCompareCommandAcceptsNTFileNames: boolean;
    fSilentModeByDefault: boolean;
    fShowDescriptionIsSyncFile: boolean;
    fGlobalRemember: boolean;
    fDisableInternalHotkeys: boolean;
    fChangeDirectionInIncompleteBlocks: boolean;
    procedure SetCompareCommand(const Value: string);
    procedure SetCompareCommandAcceptsNTFileNames(const Value: boolean);
    procedure SetCompareCommandArguments(const Value: string);
    procedure SetCompareContents(const Value: boolean);
    procedure SetCompareContentsIgnore(const Value: boolean);
    procedure SetCompareFileNameCase(const Value: boolean);
    procedure SetCompareSize(const Value: boolean);
    procedure SetCompareTime(const Value: boolean);
    procedure SetDisableInternalHotkeys(const Value: boolean);
    procedure SetGlobalRemember(const Value: boolean);
    procedure SetChangeDirectionInIncompleteBlocks(const Value: boolean);
    procedure SetIgnoreNewLines(const Value: boolean);
    procedure SetIgnorePossibleTimeZoneDifferences(const Value: boolean);
    procedure SetIgnoreWhitespace(const Value: boolean);
    procedure SetLowPrecisionTime(const Value: boolean);
    procedure SetMaxScanDepth(const Value: integer);
    procedure SetProcessSelected(const Value: boolean);
    procedure SetProcessSubfolders(const Value: boolean);
    procedure SetSelectDifferencesOnBothPanels(const Value: boolean);
    procedure SetShowDescriptionIsSyncFile(const Value: boolean);
    procedure SetSilentModeByDefault(const Value: boolean);
    procedure SetUseMaxScanDepth(const Value: boolean);
  protected
    property Changed: boolean read fChanged;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Init;
    procedure Load;
    procedure Save(Force: boolean = False);
    procedure Assign(Options: TDirSyncOptions);
    property ProcessSubfolders: boolean read fProcessSubfolders write SetProcessSubfolders;
    property UseMaxScanDepth: boolean read fUseMaxScanDepth write SetUseMaxScanDepth;
    property MaxScanDepth: integer read fMaxScanDepth write SetMaxScanDepth;
    property ProcessSelected: boolean read fProcessSelected write SetProcessSelected;
    property CompareTime: boolean read fCompareTime write SetCompareTime;
    property LowPrecisionTime: boolean read fLowPrecisionTime write SetLowPrecisionTime;
    property IgnorePossibleTimeZoneDifferences: boolean read fIgnorePossibleTimeZoneDifferences write SetIgnorePossibleTimeZoneDifferences;
    property CompareSize: boolean read fCompareSize write SetCompareSize;
    property CompareFileNameCase: boolean read fCompareFileNameCase write SetCompareFileNameCase;
    property CompareContents: boolean read fCompareContents write SetCompareContents;
    property CompareContentsIgnore: boolean read fCompareContentsIgnore write SetCompareContentsIgnore;
    property IgnoreNewLines: boolean read fIgnoreNewLines write SetIgnoreNewLines;
    property IgnoreWhitespace: boolean read fIgnoreWhitespace write SetIgnoreWhitespace;
    property SelectDifferencesOnBothPanels: boolean read fSelectDifferencesOnBothPanels write SetSelectDifferencesOnBothPanels;
    property CompareCommand: string read fCompareCommand write SetCompareCommand;
    property CompareCommandArguments: string read fCompareCommandArguments write SetCompareCommandArguments;
    property CompareCommandAcceptsNTFileNames: boolean read fCompareCommandAcceptsNTFileNames write SetCompareCommandAcceptsNTFileNames;
    property SilentModeByDefault: boolean read fSilentModeByDefault write SetSilentModeByDefault;
    property ShowDescriptionIsSyncFile: boolean read fShowDescriptionIsSyncFile write SetShowDescriptionIsSyncFile;
    property GlobalRemember: boolean read fGlobalRemember write SetGlobalRemember;
    property DisableInternalHotkeys: boolean read fDisableInternalHotkeys write SetDisableInternalHotkeys;
    property ChangeDirectionInIncompleteBlocks: boolean read fChangeDirectionInIncompleteBlocks write SetChangeDirectionInIncompleteBlocks;
  end;

implementation

{ TDirSyncOptions }

procedure TDirSyncOptions.Assign(Options: TDirSyncOptions);
begin
  ProcessSubfolders := Options.ProcessSubfolders;
  UseMaxScanDepth := Options.UseMaxScanDepth;
  MaxScanDepth := Options.MaxScanDepth;
  ProcessSelected := Options.ProcessSelected;
  CompareTime := Options.CompareTime;
  LowPrecisionTime := Options.LowPrecisionTime;
  IgnorePossibleTimeZoneDifferences := Options.IgnorePossibleTimeZoneDifferences;
  CompareSize := Options.CompareSize;
  CompareFileNameCase := Options.CompareFileNameCase;
  CompareContents := Options.CompareContents;
  CompareContentsIgnore := Options.CompareContentsIgnore;
  IgnoreNewLines := Options.IgnoreNewLines;
  IgnoreWhitespace := Options.IgnoreWhitespace;
  SelectDifferencesOnBothPanels := Options.SelectDifferencesOnBothPanels;
  CompareCommand := Options.CompareCommand;
  CompareCommandArguments := Options.CompareCommandArguments;
  CompareCommandAcceptsNTFileNames := Options.CompareCommandAcceptsNTFileNames;
  SilentModeByDefault := Options.SilentModeByDefault;
  ShowDescriptionIsSyncFile := Options.ShowDescriptionIsSyncFile;
  GlobalRemember := Options.GlobalRemember;
  DisableInternalHotkeys := Options.DisableInternalHotkeys;
  ChangeDirectionInIncompleteBlocks := Options.ChangeDirectionInIncompleteBlocks;
end;

constructor TDirSyncOptions.Create;
begin
  inherited Create('DirSync', False);
  Init;
end;

destructor TDirSyncOptions.Destroy;
begin
  inherited;
end;

procedure TDirSyncOptions.Init;
begin
  ProcessSubfolders := False;
  UseMaxScanDepth := False;
  MaxScanDepth := 99;
  ProcessSelected := False;
  CompareTime := True;
  LowPrecisionTime := True;
  IgnorePossibleTimeZoneDifferences := True;
  CompareSize := True;
  CompareFileNameCase := False;
  CompareContents := False;
  CompareContentsIgnore := False;
  IgnoreNewLines := True;
  IgnoreWhitespace := False;
  SelectDifferencesOnBothPanels := False;
  CompareCommand := 'winmerge';
  CompareCommandArguments := '${LEFTFILE} ${RIGHTFILE}';
  CompareCommandAcceptsNTFileNames := False; // WinMerge does not support them
  SilentModeByDefault := False;
  ShowDescriptionIsSyncFile := True;
  GlobalRemember := False;
  DisableInternalHotkeys := False;
  ChangeDirectionInIncompleteBlocks := False;
end;

procedure TDirSyncOptions.Load;
begin
  if Open then
    try
      ProcessSubfolders := ReadBoolean('ProcessSubfolders', ProcessSubfolders);
      UseMaxScanDepth := ReadBoolean('UseMaxScanDepth', UseMaxScanDepth);
      MaxScanDepth := ReadInteger('MaxScanDepth', MaxScanDepth);
      ProcessSelected := ReadBoolean('ProcessSelected', ProcessSelected);
      CompareTime := ReadBoolean('CompareTime', CompareTime);
      LowPrecisionTime := ReadBoolean('LowPrecisionTime', LowPrecisionTime);
      IgnorePossibleTimeZoneDifferences := ReadBoolean('IgnorePossibleTimeZoneDifferences', IgnorePossibleTimeZoneDifferences);
      CompareSize := ReadBoolean('CompareSize', CompareSize);
      CompareFileNameCase := ReadBoolean('CompareFileNameCase', CompareFileNameCase);
      CompareContents := ReadBoolean('CompareContents', CompareContents);
      CompareContentsIgnore := ReadBoolean('CompareContentsIgnore', CompareContentsIgnore);
      IgnoreNewLines := ReadBoolean('IgnoreNewLines', IgnoreNewLines);
      IgnoreWhitespace := ReadBoolean('IgnoreWhitespace', IgnoreWhitespace);
      SelectDifferencesOnBothPanels := ReadBoolean('SelectDifferencesOnBothPanels', SelectDifferencesOnBothPanels);
      CompareCommand := ReadString('CompareCommand', CompareCommand);
      CompareCommandArguments := ReadString('CompareCommandArguments', CompareCommandArguments);
      CompareCommandAcceptsNTFileNames := ReadBoolean('CompareCommandAcceptsNTFileNames', CompareCommandAcceptsNTFileNames);
      SilentModeByDefault := ReadBoolean('SilentModeByDefault', SilentModeByDefault);
      ShowDescriptionIsSyncFile := ReadBoolean('ShowDescriptionIsSyncFile', ShowDescriptionIsSyncFile);
      GlobalRemember := ReadBoolean('GlobalRemember', GlobalRemember);
      DisableInternalHotkeys := ReadBoolean('DisableInternalHotkeys', DisableInternalHotkeys);
      ChangeDirectionInIncompleteBlocks := ReadBoolean('ChangeDirectionInIncompleteBlocks', ChangeDirectionInIncompleteBlocks);
      fChanged := False;
    finally
      Close;
    end;
end;

procedure TDirSyncOptions.Save(Force: boolean);
begin
  if not (Force or Changed) then
    Exit;
  if Open then
    try
      WriteBoolean('ProcessSubfolders', ProcessSubfolders);
      WriteBoolean('UseMaxScanDepth', UseMaxScanDepth);
      WriteInteger('MaxScanDepth', MaxScanDepth);
      WriteBoolean('ProcessSelected', ProcessSelected);
      WriteBoolean('CompareTime', CompareTime);
      WriteBoolean('LowPrecisionTime', LowPrecisionTime);
      WriteBoolean('IgnorePossibleTimeZoneDifferences', IgnorePossibleTimeZoneDifferences);
      WriteBoolean('CompareSize', CompareSize);
      WriteBoolean('CompareFileNameCase', CompareFileNameCase);
      WriteBoolean('CompareContents', CompareContents);
      WriteBoolean('CompareContentsIgnore', CompareContentsIgnore);
      WriteBoolean('IgnoreNewLines', IgnoreNewLines);
      WriteBoolean('IgnoreWhitespace', IgnoreWhitespace);
      WriteBoolean('SelectDifferencesOnBothPanels', SelectDifferencesOnBothPanels);
      WriteString('CompareCommand', CompareCommand);
      WriteString('CompareCommandArguments', CompareCommandArguments);
      WriteBoolean('CompareCommandAcceptsNTFileNames', CompareCommandAcceptsNTFileNames);
      WriteBoolean('SilentModeByDefault', SilentModeByDefault);
      WriteBoolean('ShowDescriptionIsSyncFile', ShowDescriptionIsSyncFile);
      WriteBoolean('GlobalRemember', GlobalRemember);
      WriteBoolean('DisableInternalHotkeys', DisableInternalHotkeys);
      WriteBoolean('ChangeDirectionInIncompleteBlocks', ChangeDirectionInIncompleteBlocks);
      fChanged := False;
    finally
      Close;
    end;
end;

procedure TDirSyncOptions.SetChangeDirectionInIncompleteBlocks(const Value: boolean);
begin
  if Value <> fChangeDirectionInIncompleteBlocks then
  begin
    fChangeDirectionInIncompleteBlocks := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetCompareCommand(const Value: string);
begin
  if Value <> fCompareCommand then
  begin
    fCompareCommand := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetCompareCommandAcceptsNTFileNames(const Value: boolean);
begin
  if Value <> fCompareCommandAcceptsNTFileNames then
  begin
    fCompareCommandAcceptsNTFileNames := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetCompareCommandArguments(const Value: string);
begin
  if Value <> fCompareCommandArguments then
  begin
    fCompareCommandArguments := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetCompareContents(const Value: boolean);
begin
  if Value <> fCompareContents then
  begin
    fCompareContents := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetCompareContentsIgnore(const Value: boolean);
begin
  if Value <> fCompareContentsIgnore then
  begin
    fCompareContentsIgnore := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetCompareFileNameCase(const Value: boolean);
begin
  if Value <> fCompareFileNameCase then
  begin
    fCompareFileNameCase := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetCompareSize(const Value: boolean);
begin
  if Value <> fCompareSize then
  begin
    fCompareSize := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetCompareTime(const Value: boolean);
begin
  if Value <> fCompareTime then
  begin
    fCompareTime := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetDisableInternalHotkeys(const Value: boolean);
begin
  if Value <> fDisableInternalHotkeys then
  begin
    fDisableInternalHotkeys := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetGlobalRemember(const Value: boolean);
begin
  if Value <> fGlobalRemember then
  begin
    fGlobalRemember := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetIgnoreNewLines(const Value: boolean);
begin
  if Value <> fIgnoreNewLines then
  begin
    fIgnoreNewLines := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetIgnorePossibleTimeZoneDifferences(const Value: boolean);
begin
  if Value <> fIgnorePossibleTimeZoneDifferences then
  begin
    fIgnorePossibleTimeZoneDifferences := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetIgnoreWhitespace(const Value: boolean);
begin
  if Value <> fIgnoreWhitespace then
  begin
    fIgnoreWhitespace := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetLowPrecisionTime(const Value: boolean);
begin
  if Value <> fLowPrecisionTime then
  begin
    fLowPrecisionTime := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetMaxScanDepth(const Value: integer);
begin
  if Value <> fMaxScanDepth then
  begin
    fMaxScanDepth := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetProcessSelected(const Value: boolean);
begin
  if Value <> fProcessSelected then
  begin
    fProcessSelected := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetProcessSubfolders(const Value: boolean);
begin
  if Value <> fProcessSubfolders then
  begin
    fProcessSubfolders := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetSelectDifferencesOnBothPanels(const Value: boolean);
begin
  if Value <> fSelectDifferencesOnBothPanels then
  begin
    fSelectDifferencesOnBothPanels := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetShowDescriptionIsSyncFile(const Value: boolean);
begin
  if Value <> fShowDescriptionIsSyncFile then
  begin
    fShowDescriptionIsSyncFile := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetSilentModeByDefault(const Value: boolean);
begin
  if Value <> fSilentModeByDefault then
  begin
    fSilentModeByDefault := Value;
    fChanged := True;
  end;
end;

procedure TDirSyncOptions.SetUseMaxScanDepth(const Value: boolean);
begin
  if Value <> fUseMaxScanDepth then
  begin
    fUseMaxScanDepth := Value;
    fChanged := True;
  end;
end;

end.
